/**
 * $Id: gtkxmlcontainer.c,v 1.2 2001/05/16 21:37:57 makki Exp $
 * $Author: makki $
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtkmenubar.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <glade/glade.h>
#include <gnome-xml/tree.h>
#include "gtkxmlcontainer.h"

extern GladeXML *gladeUI;

struct _GtkXmlContainerPriv{
	GtkMenuBar* menubar;
	GtkToolbar* toolbar;
};
typedef struct _GtkXmlContainerPriv GtkXmlContainerPriv;

enum{
	DOCUMENT_CHANGED,
	NODE_CHANGED,
	VIEW_CHANGED,
	EDITOR_CHANGED,
	SIGNAL_NUM
};

static void gtk_xml_container_class_init( GtkXmlContainerClass *klass);
static void gtk_xml_container_init( GtkXmlContainer *container);
static void gtk_xml_container_destroy( GtkObject *object);

static GtkBoxClass *parent_class = NULL;
static guint xml_container_signals [SIGNAL_NUM] = {0};

guint
gtk_xml_container_get_type(void){
	static guint gtk_xml_container_type = 0;
	if(!gtk_xml_container_type){
		static const GtkTypeInfo gtk_xml_container_info = {
			"GtkXmlContainer",
			sizeof (GtkXmlContainer),
			sizeof (GtkXmlContainerClass),
			(GtkClassInitFunc) gtk_xml_container_class_init,
			(GtkObjectInitFunc) gtk_xml_container_init,
			NULL,
			NULL,
			(GtkClassInitFunc) NULL
		};
		gtk_xml_container_type = gtk_type_unique(gtk_hbox_get_type(), &gtk_xml_container_info);
	}
	return gtk_xml_container_type;
}

static void
gtk_xml_container_class_init(GtkXmlContainerClass* klass){
	GtkObjectClass* object_class;
	object_class = (GtkObjectClass*)klass;
	parent_class = gtk_type_class(GTK_TYPE_HBOX);
	object_class->destroy = gtk_xml_container_destroy;
	xml_container_signals[NODE_CHANGED] = gtk_signal_new("node_changed",
												GTK_RUN_FIRST,
												object_class->type,
												GTK_SIGNAL_OFFSET(GtkXmlContainerClass, node_changed),
												gtk_marshal_NONE__POINTER,
												GTK_TYPE_NONE, 1,
												GTK_TYPE_POINTER);
	xml_container_signals[DOCUMENT_CHANGED] = gtk_signal_new("document_changed",
												GTK_RUN_FIRST,
												object_class->type,
												GTK_SIGNAL_OFFSET(GtkXmlContainerClass, document_changed),
												gtk_marshal_NONE__POINTER,
												GTK_TYPE_NONE, 1,
												GTK_TYPE_POINTER);
	xml_container_signals[EDITOR_CHANGED] = gtk_signal_new("editor_changed",
												GTK_RUN_FIRST,
												object_class->type,
												GTK_SIGNAL_OFFSET(GtkXmlContainerClass, editor_changed),
												gtk_marshal_NONE__POINTER,
												GTK_TYPE_NONE, 1,
												GTK_TYPE_POINTER);
	xml_container_signals[VIEW_CHANGED] = gtk_signal_new("view_changed",
												GTK_RUN_FIRST,
												object_class->type,
												GTK_SIGNAL_OFFSET(GtkXmlContainerClass, view_changed),
												gtk_marshal_NONE__POINTER,
												GTK_TYPE_NONE, 1,
												GTK_TYPE_POINTER );
	gtk_object_class_add_signals(object_class, xml_container_signals, SIGNAL_NUM);
	klass->document_changed = NULL;
	klass->node_changed = NULL;
	klass->view_changed = NULL;
	klass->editor_changed = NULL;
}

static void
gtk_xml_container_init( GtkXmlContainer* container ){
	GtkXmlContainerPriv* priv;
	GTK_WIDGET_SET_FLAGS(GTK_WIDGET(container), GTK_NO_WINDOW);
	priv = (GtkXmlContainerPriv*)g_new0( GtkXmlContainerPriv, 1 );
	container->modules = NULL;
	container->editor = NULL;
	container->view = NULL;
	container->doc = NULL;
	container->private_data = priv;
}

GtkWidget*
gtk_xml_container_new( GtkXmlModules* modules ){
	GtkXmlContainer* container;
	container = gtk_type_new(gtk_xml_container_get_type());
	gtk_widget_set_usize(GTK_WIDGET(container),400,350);
	container->modules = modules;
	return GTK_WIDGET(container);
}

void
gtk_xml_container_destroy( GtkObject* object ){
	GtkXmlContainer *container;

	g_return_if_fail(object);
	container = GTK_XML_CONTAINER(object);
	if( GTK_OBJECT_CLASS(parent_class)->destroy){
		(*GTK_OBJECT_CLASS(parent_class)->destroy)(object);
	}
}
xmlDoc*
gtk_xml_container_get_document( GtkXmlContainer *container ){
	if( container == NULL ){
		return NULL;
	}
	/*
	if( container->view != NULL ){
		return gtk_xml_view_get_document( GTK_XML_VIEW(container->view));
	}
	return NULL;
	*/
	return container->doc;
}
xmlDoc*
gtk_xml_container_set_document( GtkXmlContainer *container, xmlDoc* doc ){
	xmlDoc* olddoc;
	gchar* mod;
	if( container == NULL ){
		return NULL;
	}
	if( doc == NULL ){
		doc = xmlNewDoc("1.0");
	}
	olddoc = gtk_xml_container_get_document(container);
	if( container->view != NULL ){
		gtk_xml_view_select_node(GTK_XML_VIEW( container->view ), NULL);
	}
	container->doc = doc;
	mod = gtk_xml_modules_find_editor( container->modules, doc );
	if( mod != NULL ){
		ModuleInfo* info = gtk_xml_modules_get_module_info( container->modules, mod );
		GtkWidget* (*func) (GtkXmlContainer*);
		if( g_module_symbol( info->module, "module_create_editor", &func ) ){
			GtkWidget* w;
			w = GTK_WIDGET( func(container) );
			gtk_xml_container_set_editor( container, w );
			mod = gtk_xml_modules_find_view( container->modules, doc );
			if( mod != NULL ){
				GtkXmlView* (*func2) (GtkXmlContainer*, GtkWidget* );
				info = gtk_xml_modules_get_module_info( container->modules, mod );
				if( g_module_symbol( info->module, "module_create_view", &func2 ) ){
					w = GTK_WIDGET( func2(container,w) );
					gtk_xml_container_set_view( container, w );
				}
			}
		}
	}
	gtk_signal_emit( GTK_OBJECT(container), xml_container_signals[DOCUMENT_CHANGED], container->doc );
	return olddoc;
}
GtkXmlView*
gtk_xml_container_get_view( GtkXmlContainer *container ){
	return (GtkXmlView *)container->view;
}
void
gtk_xml_container_set_view( GtkXmlContainer* container, GtkXmlView* view ){
	g_assert(container);
	g_assert(view);
	container->view = view;
	gtk_signal_emit( GTK_OBJECT(container), xml_container_signals[VIEW_CHANGED], container->view );
}
GtkWidget*
gtk_xml_container_get_editor( GtkXmlContainer *container ){
	g_assert(container);
	return (GtkXmlView *)container->editor;
}
void
gtk_xml_container_set_editor( GtkXmlContainer* container, GtkWidget* editor ){
	g_assert(container);
	if( container->editor != NULL ){
		gtk_container_remove( GTK_CONTAINER(container), container->editor );
	}
	g_assert(editor);
	container->editor = editor;
	gtk_container_add( GTK_CONTAINER(container), container->editor );
	gtk_signal_emit( GTK_OBJECT(container), xml_container_signals[EDITOR_CHANGED], editor );
	gtk_widget_show( container->editor );
}

xmlNode*
gtk_xml_container_get_current_node( GtkXmlContainer* container ){
	g_assert(container);
	return gtk_xml_view_get_selected_node( GTK_XML_VIEW(container->view));
}
GtkXmlModules*
gtk_xml_container_get_modules( GtkXmlContainer* container ){
	if( container == NULL ){
		return NULL;
	}
	if( !IS_GTK_XML_CONTAINER(container) ){
		return NULL;
	}
	return GTK_XML_MODULES( container->modules );
}


void
gtk_xml_container_set_menubar( GtkXmlContainer* container, GtkMenuBar* menubar ){
	GtkXmlContainerPriv* priv = (GtkXmlContainerPriv*)container->private_data;
	priv->menubar = menubar;
}
void
gtk_xml_container_set_toolbar( GtkXmlContainer* container, GtkToolbar* toolbar ){
	GtkXmlContainerPriv* priv = (GtkXmlContainerPriv*)container->private_data;
	priv->toolbar = toolbar;
}
/** User interface
 */
GtkMenuBar*
gtk_xml_container_get_menubar( GtkXmlContainer* container ){
	GtkXmlContainerPriv* priv = (GtkXmlContainerPriv*)container->private_data;
	return priv->menubar;
}
GtkToolbar*
gtk_xml_container_get_toolbar( GtkXmlContainer* container ){
	GtkXmlContainerPriv* priv = (GtkXmlContainerPriv*)container->private_data;
	return priv->toolbar;
}
