/**
 * $Id: gtkxmlmodules.c,v 1.4 2001/05/16 21:37:57 makki Exp $
 * $Author: makki $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif



#include "gtkxmlmodules.h"
#include "modules.h"
#include <gtk/gtk.h>
#include <glade/glade.h>

typedef struct _GtkXmlModulesPrivate GtkXmlModulesPrivate;

#define GTK_XML_MODULES_PRIV(obj) ((GtkXmlModulesPrivate*)obj->private_data)



struct _GtkXmlModulesPrivate{
	ModuleList* list;
	GtkWidget* config;
	GtkWidget* config_module_list;
}_GtkXmlModulesPrivate;

enum{
	ADD_MODULE,
	REMOVE_MODULE,
	ACTIVATE_MODULE,
	DESACTIVATE_MODULE,
	SIGNAL_NUM
};
guint gtk_xml_modules_get_type(void);

static void gtk_xml_modules_class_init( GtkXmlModulesClass *klass );
static void gtk_xml_modules_init( GtkXmlModules* modules );
static void gtk_xml_modules_destroy( GtkObject* object );

static void _modules_close_sig( GtkWidget* widget, gpointer data );
static void _modules_config_append_name( GtkXmlModules* modules, char* name );

static GtkObject *parent_class = NULL;
static guint xml_modules_signals[SIGNAL_NUM] = {0};

guint
gtk_xml_modules_get_type(void){
	static guint gtk_xml_modules_type = 0;
	if(!gtk_xml_modules_type){
		static const GtkTypeInfo gtk_xml_modules_info = {
			"GtkXmlModules",
			sizeof(GtkXmlModules),
			sizeof(GtkXmlModulesClass),
			(GtkClassInitFunc)gtk_xml_modules_class_init,
			(GtkObjectInitFunc)gtk_xml_modules_init,
			NULL,
			NULL,
			(GtkClassInitFunc) NULL
		};
		gtk_xml_modules_type = gtk_type_unique(gtk_object_get_type(), &gtk_xml_modules_info);
	}
	return gtk_xml_modules_type;
}
static void
gtk_xml_modules_class_init(GtkXmlModulesClass* klass){
	GtkObjectClass* object_class;
	object_class = (GtkObjectClass*)klass;
	parent_class = gtk_type_class(GTK_TYPE_OBJECT);
	object_class->destroy = gtk_xml_modules_destroy;
	/** TODO add signals */
}

static void
gtk_xml_modules_init( GtkXmlModules* modules ){
	/** TODO */
	GtkXmlModulesPrivate* priv;
	priv = g_new0( GtkXmlModulesPrivate, 1);
	modules->private_data = priv;
	priv->list = mod_new();
}
static void
gtk_xml_modules_destroy( GtkObject* object ){
	GtkXmlModules* modules;
	g_return_if_fail(object);
	g_return_if_fail(IS_GTK_XML_MODULES(object));
	modules = GTK_XML_MODULES(object);
/*
	g_free(modules->private_data);
*/
	if( GTK_OBJECT_CLASS(parent_class)->destroy){
		(*GTK_OBJECT_CLASS(parent_class)->destroy)(object);
	}
}

GtkObject*
gtk_xml_modules_new(){
	GtkXmlModules* modules;

	modules = gtk_type_new(gtk_xml_modules_get_type());
	return GTK_OBJECT(modules);
}

void
gtk_xml_modules_load_list( GtkXmlModules* modules, xmlDoc* doc ){
	mod_load_list( GTK_XML_MODULES_PRIV(modules)->list, doc );
}

gboolean
gtk_xml_modules_add_module( GtkXmlModules* modules, const gchar* filename ){
	return mod_add_module( GTK_XML_MODULES_PRIV(modules)->list, filename ) != NULL;
}
GModule*
gtk_xml_modules_get_module( GtkXmlModules* modules, const gchar* name ){
	ModuleInfo* info = gtk_xml_modules_get_module_info( modules, name );
	if( info != NULL ){
		return info->module;
	}
	return NULL;
}
gchar*
gtk_xml_modules_find_editor( GtkXmlModules* modules, xmlDocPtr doc ){
	GList* list = GTK_XML_MODULES_PRIV(modules)->list->list;
	ModuleInfo* info;
	while( list != NULL ){
		info = (ModuleInfo*)list->data;
		if( (info->type & MODULE_EDITOR)==MODULE_EDITOR){
			return info->name;
		}
		list = g_list_next(list);
	}
	show_error("Editor module not found. Probably a configuration error or the needed module is not enabled.\n");
	return NULL;
}
gchar*
gtk_xml_modules_find_view( GtkXmlModules* modules, xmlDocPtr doc ){
	GList* list = GTK_XML_MODULES_PRIV(modules)->list->list;
	ModuleInfo* info;
	while( list != NULL ){
		info = (ModuleInfo*)list->data;
		if( (info->type & MODULE_VIEW)==MODULE_VIEW){
			return info->name;
		}
		list = g_list_next(list);
	}
	show_error("View module not found. Probably a configuration error or the needed module is not enabled.\n");
	return NULL;
}

ModuleInfo*
gtk_xml_modules_get_module_info( GtkXmlModules* modules, const gchar* name ){
	return (ModuleInfo*)g_hash_table_lookup( GTK_XML_MODULES_PRIV(modules)->list->table, name );
}

gboolean
gtk_xml_modules_is_import( GtkXmlModules* modules, const gchar* name ){
	ModuleInfo* info = gtk_xml_modules_get_module_info( modules, name );
	if( info != NULL ){
		return (info->type & MODULE_IMPORT) == MODULE_IMPORT;
	}
	return FALSE;
}
gboolean
gtk_xml_modules_is_export( GtkXmlModules* modules, const gchar* name ){
	ModuleInfo* info = gtk_xml_modules_get_module_info( modules, name );
	if( info != NULL ){
		return (info->type & MODULE_IMPORT) == MODULE_EXPORT;
	}
	return FALSE;
}
gboolean
gtk_xml_modules_is_editor( GtkXmlModules* modules, const gchar* name ){
	ModuleInfo* info = gtk_xml_modules_get_module_info( modules, name );
	if( info != NULL ){
		return (info->type & MODULE_IMPORT) == MODULE_EDITOR;
	}
	return FALSE;
}
gboolean
gtk_xml_modules_is_view( GtkXmlModules* modules, const gchar* name ){
	ModuleInfo* info = gtk_xml_modules_get_module_info( modules, name );
	if( info != NULL ){
		return (info->type & MODULE_IMPORT) == MODULE_VIEW;
	}
	return FALSE;
}
GtkWidget*
gtk_xml_modules_get_configuration( GtkXmlModules* m ){
	GtkXmlModulesPrivate* priv = GTK_XML_MODULES_PRIV(m);
	if( priv->config == NULL ){
		GList* list;
		GladeXML* xml = glade_xml_new(GLADE_UI_FILE,"modules");
		if( xml == NULL ){
			g_print("ERROR: glade widget \"modules\" not found in %s\n",GLADE_UI_FILE);
			return NULL;
		}
		list = GTK_XML_MODULES_PRIV(m)->list->list;
		glade_xml_signal_autoconnect(xml);
		priv->config_module_list = glade_xml_get_widget(xml,"module_list");
		if( priv->config_module_list == NULL ){
			g_print("ERROR: widget \"module_list\" not found in %s:modules\n",GLADE_UI_FILE);
			return NULL;
		}
		while( list != NULL ){
			ModuleInfo* info = (ModuleInfo*)list->data;
			if( info == NULL ){
				return NULL;
			}
			_modules_config_append_name(m, info->name );
			list = g_list_next(list);
		}
		priv->config = glade_xml_get_widget(xml,"modules");
		gtk_signal_connect(GTK_OBJECT(priv->config),"destroy",GTK_SIGNAL_FUNC(_modules_close_sig), m );
		gtk_object_set_user_data(GTK_OBJECT(priv->config), (gpointer)m );
	}
	return priv->config;
}

/**
 * Signals
 */

static void _modules_filedlg_ok_callback( GtkWidget* widget, char** n );

glade_modules_toggle( GtkToggleButton* button, gpointer user_data ){
	GtkWidget* widget = gtk_widget_get_toplevel( GTK_WIDGET(button) );
	GtkXmlModules* modules = GTK_XML_MODULES(gtk_object_get_user_data(GTK_OBJECT(widget)));

}

void
glade_modules_add_clicked( GtkButton* button, gpointer user_data ){
	GtkWidget* widget = gtk_widget_get_toplevel(GTK_WIDGET(button));
	char* filename;
	GladeXML* xml = glade_xml_new(GLADE_UI_FILE,"fileopen");
	GtkXmlModules* modules = GTK_XML_MODULES(gtk_object_get_user_data(GTK_OBJECT(widget)));
	glade_xml_signal_autoconnect(xml);
	widget = glade_xml_get_widget(xml, "fileopen");
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(widget)->ok_button),
								"clicked", _modules_filedlg_ok_callback, &filename );
	gtk_widget_show(widget);
	gtk_main();
	if( gtk_xml_modules_get_module(modules,filename) == NULL ){
		if( gtk_xml_modules_add_module(modules, filename ) ){
			_modules_config_append_name(modules, filename );
		}
	}
	/*
	g_free(filename);
	*/
	gtk_widget_destroy(widget);
}
static void
_modules_filedlg_ok_callback( GtkWidget* widget, char** n ){
	GtkWidget* s = gtk_widget_get_toplevel(widget);
	((char*)*n) = (char*)strdup( gtk_file_selection_get_filename( GTK_FILE_SELECTION(s)));
	gtk_main_quit();
}
void
_modules_close_sig( GtkWidget* widget, gpointer data ){
	GtkXmlModules* mod = GTK_XML_MODULES(data);
	GTK_XML_MODULES_PRIV(mod)->config = NULL;
}
static void
_modules_config_append_name( GtkXmlModules* modules, char* name ){
	gtk_clist_append( GTK_CLIST( GTK_XML_MODULES_PRIV(modules)->config_module_list), &name );
}
GList*
gtk_xml_modules_get_module_names( GtkXmlModules* modules ){
	GtkXmlModulesPrivate* priv;
	GList* l = NULL;
	GList* l2;
	ModuleInfo* info;
	g_assert(modules);
	priv = (GtkXmlModulesPrivate*)modules->private_data;
	if( priv == NULL ){
		return l;
	}
	if( priv->list == NULL ){
		return l;
	}
	l2 = priv->list->list;
	while( l2 != NULL ){
		info = (ModuleInfo*)l2->data;
		l = g_list_append(l,info->name);
		l2 = g_list_next(l2);
	}
	return l;
}
