/**
 * $Id: gtkxmltree.c,v 1.8 2001/05/16 21:37:57 makki Exp $
 * $Author: makki $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gtkxmltree.h"
#include <gtk/gtk.h>
#include <gtk/gtktree.h>
#include <glade/glade.h>
#include "gtkxmlview.h"

struct _GtkXmlTreePrivate{
	GtkTree* tree;
	GtkWidget *popup_menu;
};
typedef struct _GtkXmlTreePrivate GtkXmlTreePrivate;
struct _GtkXmlTreeContext{
	GtkXmlTree* tree;
	xmlNode* context;
};
typedef struct _GtkXmlTreeContext GtkXmlTreeContext;

enum{
	SIGNAL_NUM
};

static void gtk_xml_tree_class_init( GtkXmlTreeClass *klass);
static void gtk_xml_tree_init( GtkXmlTree *tree);
static void gtk_xml_tree_destroy( GtkObject *object);
static void _tree_select_node( GtkXmlTree* tree, xmlNode* node, gpointer data );
void _tree_append_child( GtkXmlTree* tree, xmlNode* node, xmlNode* parent );
void _tree_delete_node( GtkXmlTree* tree, xmlNode* node );
static void _tree_select_child( GtkWidget* tree, GtkWidget* child, gpointer data );
static void _tree_unselect_child( GtkWidget* tree, GtkWidget* child, gpointer data );
static void _tree_update_node( GtkXmlView* view, xmlNode* node, gpointer user_data );
static gint _tree_menu_button_press(GtkWidget* widget, GdkEvent* event, gpointer user_data);

GtkWidget* _tree_search_node_widget(GtkXmlTree* tree, xmlNode* node);
GtkWidget* _search_sub_node_widget( GtkWidget* tree, xmlNode* node );

static GtkXmlViewClass *parent_class = NULL;
static guint xml_tree_signals [SIGNAL_NUM] = {0};
GtkXmlTreeContext* _tree_context_create( GtkXmlTree* tree, xmlNode* context );
GtkXmlTreeContext* _tree_context_free( GtkXmlTreeContext* ctx );

guint
gtk_xml_tree_get_type(void){
	static guint gtk_xml_tree_type = 0;
	if(!gtk_xml_tree_type){
		static const GtkTypeInfo gtk_xml_tree_info = {
			"GtkXmlTree",
			sizeof (GtkXmlTree),
			sizeof (GtkXmlTreeClass),
			(GtkClassInitFunc) gtk_xml_tree_class_init,
			(GtkObjectInitFunc) gtk_xml_tree_init,
			NULL,
			NULL,
			(GtkClassInitFunc) NULL
		};
		gtk_xml_tree_type = gtk_type_unique(gtk_xml_view_get_type(), &gtk_xml_tree_info);
	}
	return gtk_xml_tree_type;
}

static void
gtk_xml_tree_class_init(GtkXmlTreeClass* klass){
	GtkObjectClass* object_class;
	object_class = (GtkObjectClass*)klass;
	parent_class = gtk_type_class(TYPE_GTK_XML_VIEW);
	object_class->destroy = gtk_xml_tree_destroy;
	gtk_object_class_add_signals( object_class, xml_tree_signals, SIGNAL_NUM );
}


static void
gtk_xml_tree_init( GtkXmlTree* tree ){
	GtkXmlTreePrivate* priv;
	
	GTK_WIDGET_SET_FLAGS(GTK_WIDGET(tree), GTK_NO_WINDOW);

	priv = g_new0( GtkXmlTreePrivate, 1);
	/*
	priv->current = NULL;
	*/
	tree->private_data = priv;
	priv->tree = GTK_TREE(gtk_tree_new());
	gtk_tree_set_selection_mode(priv->tree, GTK_SELECTION_BROWSE);
	gtk_box_pack_start_defaults(GTK_BOX(tree), GTK_WIDGET(priv->tree));
	GTK_WIDGET_SET_FLAGS( priv->tree, GTK_CAN_FOCUS );
	gtk_signal_connect( GTK_OBJECT(priv->tree), "select_child", GTK_SIGNAL_FUNC(_tree_select_child), tree);
	gtk_signal_connect( GTK_OBJECT(priv->tree), "unselect_child", GTK_SIGNAL_FUNC(_tree_unselect_child), tree);
	gtk_signal_connect( GTK_OBJECT(tree), "update_node", GTK_SIGNAL_FUNC(_tree_update_node), tree );
	gtk_signal_connect( GTK_OBJECT(tree), "append_child_node", GTK_SIGNAL_FUNC(_tree_append_child), tree );
	gtk_signal_connect( GTK_OBJECT(tree), "delete_node", GTK_SIGNAL_FUNC(_tree_delete_node), tree);
	gtk_signal_connect( GTK_OBJECT(tree), "select_node", GTK_SIGNAL_FUNC(_tree_select_node), tree);
	gtk_widget_show( GTK_WIDGET(priv->tree));
}

GtkWidget*
gtk_xml_tree_new(xmlDoc* doc){
	GtkXmlTree* tree;
	xmlNode* root;
	tree = gtk_type_new(gtk_xml_tree_get_type());
/*
	gtk_widget_set_usize(GTK_WIDGET(tree),100,50);
*/
	if( doc == NULL ){
		doc = xmlNewDoc("1.0");
	}
	root = doc->root;
	if( root != NULL ){
		gtk_xml_view_append_child( GTK_XML_VIEW(tree), root, doc );
	}else{
		GTK_XML_VIEW(tree)->doc = doc;
	}
	return GTK_WIDGET(tree);
}
void
gtk_xml_tree_destroy( GtkObject* object ){
	GtkXmlTree *tree;
	g_return_if_fail(object);
	g_return_if_fail(IS_GTK_XML_TREE(object));
	tree = GTK_XML_TREE(object);
	g_free(tree->private_data);
	if( GTK_OBJECT_CLASS(parent_class)->destroy){
		(*GTK_OBJECT_CLASS(parent_class)->destroy)(object);
	}
}
static void
_tree_select_node( GtkXmlTree* tree, xmlNode* node, gpointer user_data ){
	if( node != NULL ){
		GtkWidget* widget = _tree_search_node_widget( GTK_XML_TREE(tree), node );
		GtkXmlTreePrivate* priv = GTK_XML_TREE(tree)->private_data;
		if( widget == NULL ){
			return;
		}
		gtk_tree_select_child( priv->tree, widget );
	}
}
static void
_tree_select_child( GtkWidget* tree, GtkWidget* child, gpointer data ){
	xmlNode* node;
	GtkXmlTreePrivate* priv = (GtkXmlTreePrivate*)GTK_XML_TREE(data)->private_data;
	if( child == NULL ){
		return;
	}
	if( ( node = gtk_xml_view_get_selected_node( GTK_XML_VIEW(data)) ) != NULL ){
		gtk_xml_view_unselect_node( GTK_XML_VIEW(data), node );
	}
	node = gtk_object_get_user_data( GTK_OBJECT(child));
	if( node != NULL ){
		gtk_signal_handler_block_by_func( GTK_OBJECT(data), _tree_select_node, data );
		gtk_xml_view_select_node( GTK_XML_VIEW(data), node );
		gtk_signal_handler_unblock_by_func( GTK_OBJECT(data), _tree_select_node, data );
	}
}
static void
_tree_unselect_child( GtkWidget* tree, GtkWidget* child, gpointer data ){
	xmlNode* node;
	if( child == NULL ){
		return;
	}
	node = gtk_object_get_user_data( GTK_OBJECT(child));
	if( node != NULL ){
		gtk_signal_emit_by_name(GTK_OBJECT(GTK_XML_VIEW(data)), 
                                        "unselect_node", node );
	}
}
static void
_tree_update_node( GtkXmlView* view, xmlNode* node, gpointer user_data ){
	GtkXmlTree* tree = GTK_XML_TREE(user_data);
	GtkWidget* widget = _tree_search_node_widget(tree, node);
	if( widget != NULL ){
		widget = GTK_BIN(widget)->child;
		if( widget != NULL ){
			if( GTK_IS_LABEL(widget) ){
				gtk_label_set_text(GTK_LABEL(widget), node->name );
			}
		}
	}
}
GtkWidget*
_tree_search_node_widget(GtkXmlTree* tree, xmlNode* node){
	GtkXmlTreePrivate* priv = (GtkXmlTreePrivate*)tree->private_data;
	GtkWidget* widget = GTK_WIDGET(priv->tree);
	if( widget == NULL ){
		return NULL;
	}
	if( node == NULL ){
		return NULL;
	}
	if( node->type == XML_DOCUMENT_NODE){
		return widget;
	}
	return _search_sub_node_widget(widget, node );
}
/**
 * This can be made much more efficient
 */
GtkWidget*
_search_sub_node_widget( GtkWidget* tree, xmlNode* node ){
	GtkWidget* widget;
	GList *item;
	if( node == NULL ){
		return NULL;
	}
	if( !GTK_IS_TREE(tree) ){
		if( gtk_object_get_user_data( GTK_OBJECT(tree)) == node ){
			return GTK_WIDGET(tree);
		}else{
			return _search_sub_node_widget( GTK_TREE_ITEM_SUBTREE(tree), node );
		}
	}
	item = gtk_container_children( GTK_CONTAINER(tree));
	while( item != NULL ){
		widget = GTK_WIDGET( item->data );
		if( gtk_object_get_user_data(GTK_OBJECT(widget) ) == node ){
			return GTK_WIDGET(widget);
		}
		if( GTK_TREE_ITEM_SUBTREE(widget) != NULL ){
			if( ( widget = _search_sub_node_widget( GTK_TREE_ITEM_SUBTREE(widget), node )) != NULL){
				return widget;
			}
		}
		item = g_list_next(item);
	}
	return NULL;
}

GtkWidget*
_create_node_widget( GtkXmlTree *obj, GtkTree* tree, xmlNode* node ){
	GtkWidget* widget;
	GtkXmlTreePrivate* priv = (GtkXmlTreePrivate*)GTK_XML_TREE(obj)->private_data;
	xmlNode* l;
	switch( node->type ){
		case XML_ELEMENT_NODE:
			widget = gtk_tree_item_new_with_label(node->name);
			gtk_tree_append( tree, widget );
			gtk_signal_connect_object_after(
						GTK_OBJECT(widget),
						"button_press_event",
						GTK_SIGNAL_FUNC(_tree_menu_button_press),
						GTK_OBJECT(obj));
			l = node->childs;
			if( l != NULL ){
				tree = GTK_TREE(gtk_tree_new());
				gtk_signal_connect_object_after(
							GTK_OBJECT(tree),
							"button_press_event",
							GTK_SIGNAL_FUNC(_tree_menu_button_press),
							GTK_OBJECT(obj) );
				gtk_tree_item_set_subtree( GTK_TREE_ITEM(widget), GTK_WIDGET(tree) );
				while( l != NULL ){
					gtk_signal_connect( GTK_OBJECT(tree), "select_child", GTK_SIGNAL_FUNC(_tree_select_child), obj);
					gtk_signal_connect( GTK_OBJECT(tree), "unselect_child", GTK_SIGNAL_FUNC(_tree_unselect_child), obj);
					gtk_widget_show( _create_node_widget( obj, tree, l ) );
					l = l->next;
				}
			}
			break;
		case XML_TEXT_NODE:
			widget = gtk_tree_item_new_with_label( node->name );
			gtk_signal_connect_object_after(
						GTK_OBJECT(widget),
						"button_press_event",
						GTK_SIGNAL_FUNC(_tree_menu_button_press),
						GTK_OBJECT(obj));
			gtk_tree_append( tree, widget );
			break;
		default:
			g_print("UNSUPPORTED NODE TYPE\n");
			return NULL;
	}
	GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_FOCUS );
	gtk_object_set_user_data( GTK_OBJECT(GTK_TREE_ITEM(widget)), node );
	return widget;
}

void
_tree_append_child( GtkXmlTree* tree, xmlNode* node, xmlNode* parent ){
	GtkWidget* widget;
	GtkWidget* parent_widget;
	if( node == NULL ){
		return;
	}
	parent_widget = _tree_search_node_widget(tree, parent);
	if( parent_widget == NULL ){
		return;
	}
	if( !GTK_IS_TREE( parent_widget ) ){
		if( GTK_TREE_ITEM_SUBTREE(parent_widget) == NULL ){
			widget = gtk_tree_new();
			gtk_tree_item_set_subtree( GTK_TREE_ITEM(parent_widget), widget );
			gtk_signal_connect( GTK_OBJECT(widget), "unselect_child", GTK_SIGNAL_FUNC(_tree_unselect_child), tree);
			gtk_signal_connect( GTK_OBJECT(widget), "select_child", GTK_SIGNAL_FUNC(_tree_select_child), tree);
			/*
			gtk_signal_connect_object_after( GTK_OBJECT(widget), "button_press_event", GTK_SIGNAL_FUNC(_tree_menu_button_press), GTK_OBJECT(obj))
			*/
			parent_widget = widget;
		}else{
			parent_widget = GTK_TREE_ITEM_SUBTREE(GTK_TREE_ITEM(parent_widget));
		}
	}
	widget = _create_node_widget(tree, GTK_TREE(parent_widget), node );
	gtk_widget_show(widget);
}
void
_tree_delete_node( GtkXmlTree* tree, xmlNode* node ){
	GtkWidget* parent;
	GtkWidget* widget;
	GtkXmlTreePrivate* priv = (GtkXmlTreePrivate*)tree->private_data;
	widget = _tree_search_node_widget(tree, node );
	if( widget == NULL ){
		return;
	}
	parent = widget->parent;
	if( GTK_IS_TREE_ITEM(parent) ){
		parent = GTK_TREE_ITEM_SUBTREE(parent);
	}
	if( priv->popup_menu != NULL ){
		GtkXmlTreeContext* ctx = (GtkXmlTreeContext*)gtk_object_get_user_data( GTK_OBJECT(priv->popup_menu) );
		if( ctx != NULL ){
			if( ctx->context == node ){
				gtk_widget_destroy( priv->popup_menu);
				priv->popup_menu = NULL;
			}
		}
	}
	gtk_container_remove( GTK_CONTAINER(parent), widget );
}
static GtkWidget*
_tree_get_context_popup_menu( GtkXmlTree* e ){
	GladeXML* gladeUI;
	GtkWidget *popup_menu;
	GtkXmlTreePrivate* priv = (GtkXmlTreePrivate*)e->private_data;
	if( priv->popup_menu == NULL ){
		gladeUI = glade_xml_new(GLADE_UI_FILE,"contextmenu");
		if( gladeUI == NULL ){
			g_print("%s not found, cannot construct context menu\n",GLADE_UI_FILE);
			return NULL;
		}
		glade_xml_signal_autoconnect(gladeUI);
		priv->popup_menu = glade_xml_get_widget(gladeUI, "contextmenu");
	}
	return priv->popup_menu;
}
static gint
_tree_menu_button_press(GtkWidget* widget, GdkEvent* event, gpointer user_data){
	GtkWidget* parent = (GtkWidget *)NULL;
	GtkXmlTreePrivate* priv = (GtkXmlTreePrivate*)GTK_XML_TREE(widget)->private_data;
	if(event->type == GDK_BUTTON_PRESS){
		if( event->button.button == 3 ){
			GdkEventButton *bevent = (GdkEventButton *) event;
			GtkMenu* menu = GTK_MENU(_tree_get_context_popup_menu(GTK_XML_TREE(widget)));
			xmlNode* node = NULL;
			GtkXmlTreeContext* ctx;
			if( GTK_IS_TREE_ITEM(user_data) ){
				node = (xmlNode*)gtk_object_get_user_data(GTK_OBJECT(GTK_TREE_ITEM(user_data)));
			}
			ctx = (GtkXmlTreeContext*)gtk_object_get_user_data(GTK_OBJECT(menu));
			if( ctx != NULL ){
				ctx->context = node;
				ctx->tree = GTK_XML_TREE(widget);
			}else{
				gtk_object_set_user_data(GTK_OBJECT(menu), (gpointer)(ctx = _tree_context_create( GTK_XML_TREE(widget), node )));
			}
			gtk_menu_popup( menu, NULL, NULL,
								NULL, NULL, bevent->button, bevent->time);
			/* Tell calling code that we have handled this event; the buck
			 * stops here.
			 */
			return TRUE;
		}else{
			return FALSE;
		}
	}
	return FALSE;
}
GtkXmlTreeContext*
_tree_context_create( GtkXmlTree* tree, xmlNode* context ){
	GtkXmlTreeContext* ctx = (GtkXmlTreeContext*)malloc(sizeof(GtkXmlTreeContext));
	ctx->tree = tree;
	ctx->context = context;
}
GtkXmlTreeContext*
_tree_context_free( GtkXmlTreeContext* ctx ){
	free(ctx);
	return NULL;
}
void
glade_xml_tree_on_contextmenu_delete( GtkWidget* emitter, gpointer data ){
	GtkXmlTreeContext* ctx = (GtkXmlTreeContext*)gtk_object_get_user_data(GTK_OBJECT(emitter->parent));
	gtk_xml_view_delete_node( ctx->tree, ctx->context );
}
void
glade_xml_tree_on_contextmenu_add_element( GtkWidget* emitter, gpointer data ){
	GtkXmlTreeContext* ctx = (GtkXmlTreeContext*)gtk_object_get_user_data(GTK_OBJECT(emitter->parent));
	xmlNode* n;
	gtk_xml_view_append_child( ctx->tree, n = xmlNewNode(NULL,"Untitled"), ctx->context );
	gtk_xml_view_select_node( GTK_XML_VIEW(ctx->tree), n );
}
void
glade_xml_tree_on_contextmenu_add_text( GtkWidget* emitter, gpointer data ){
	xmlNode* n;
	GtkXmlTreeContext* ctx = (GtkXmlTreeContext*)gtk_object_get_user_data(GTK_OBJECT(emitter->parent));
	gtk_xml_view_append_child( GTK_XML_VIEW(ctx->tree), n = xmlNewText(""), ctx->context );
	gtk_xml_view_select_node( GTK_XML_VIEW(ctx->tree), n );
}

