/**
 * $Id: gtkxmlview.c,v 1.5 2001/05/16 21:37:57 makki Exp $
 * $Author: makki $
 */

#include "gtkxmlview.h"
#include <gtk/gtk.h>

struct _GtkXmlViewPrivate{
	/*
	xmlDoc* doc;
	*/
};
typedef struct _GtkXmlViewPrivate GtkXmlViewPrivate;

enum{
	UPDATE_NODE,
	SELECT_NODE,
	UNSELECT_NODE,
	CLICKED,
	DOUBLECLICKED,
	APPEND_CHILD_NODE,
	DELETE_NODE,
	SIGNAL_NUM
};


guint gtk_xml_view_get_type(void);


static void gtk_xml_view_class_init( GtkXmlViewClass *klass);
static void gtk_xml_view_init( GtkXmlView *view);
void gtk_xml_view_update_node( GtkXmlView* view, xmlNode *node);
static void gtk_xml_view_clicked( GtkXmlView* view, xmlNode *node);
static void gtk_xml_view_doubleclicked( GtkXmlView* view, xmlNode *node);
static void gtk_xml_view_destroy( GtkObject *object);

static GtkBoxClass *parent_class = NULL;
static guint xml_view_signals [SIGNAL_NUM] = {0};

guint
gtk_xml_view_get_type(void){
	static guint gtk_xml_view_type = 0;
	if(!gtk_xml_view_type){
		static const GtkTypeInfo gtk_xml_view_info = {
			"GtkXmlView",
			sizeof (GtkXmlView),
			sizeof (GtkXmlViewClass),
			(GtkClassInitFunc) gtk_xml_view_class_init,
			(GtkObjectInitFunc) gtk_xml_view_init,
			NULL,
			NULL,
			(GtkClassInitFunc) NULL
		};
		gtk_xml_view_type = gtk_type_unique(gtk_hbox_get_type(), &gtk_xml_view_info);
	}
	return gtk_xml_view_type;
}

static void
gtk_xml_view_class_init(GtkXmlViewClass* klass){
	GtkObjectClass* object_class;
	object_class = (GtkObjectClass*)klass;
	parent_class = gtk_type_class(GTK_TYPE_HBOX);
	object_class->destroy = gtk_xml_view_destroy;
	xml_view_signals[UPDATE_NODE] = gtk_signal_new("update_node",
												GTK_RUN_FIRST,
												object_class->type,
												GTK_SIGNAL_OFFSET(GtkXmlViewClass, update_node),
												gtk_marshal_NONE__POINTER,
												GTK_TYPE_NONE, 1,
												GTK_TYPE_POINTER);
	xml_view_signals[SELECT_NODE] = gtk_signal_new("select_node",
												GTK_RUN_FIRST,
												object_class->type,
												GTK_SIGNAL_OFFSET(GtkXmlViewClass, select_node),
												gtk_marshal_NONE__POINTER,
												GTK_TYPE_NONE, 1,
												GTK_TYPE_POINTER);
	xml_view_signals[UNSELECT_NODE] = gtk_signal_new("unselect_node",
												GTK_RUN_FIRST,
												object_class->type,
												GTK_SIGNAL_OFFSET(GtkXmlViewClass, unselect_node),
												gtk_marshal_NONE__NONE,
												GTK_TYPE_NONE, 1,
												GTK_TYPE_POINTER);
	xml_view_signals[CLICKED] = gtk_signal_new("clicked",
												GTK_RUN_FIRST,
												object_class->type,
												GTK_SIGNAL_OFFSET(GtkXmlViewClass, clicked),
												gtk_marshal_NONE__NONE,
												GTK_TYPE_NONE, 1,
												GTK_TYPE_POINTER);
	xml_view_signals[DOUBLECLICKED] = gtk_signal_new("doubleclicked",
												GTK_RUN_FIRST,
												object_class->type,
												GTK_SIGNAL_OFFSET(GtkXmlViewClass, doubleclicked),
												gtk_marshal_NONE__NONE,
												GTK_TYPE_NONE, 1,
												GTK_TYPE_POINTER);
	xml_view_signals[APPEND_CHILD_NODE] = gtk_signal_new("append_child_node",
												GTK_RUN_FIRST,
												object_class->type,
												GTK_SIGNAL_OFFSET(GtkXmlViewClass, append_child_node),
												gtk_marshal_NONE__POINTER_POINTER,
												GTK_TYPE_NONE, 2,
												GTK_TYPE_POINTER,
												GTK_TYPE_POINTER );
	xml_view_signals[DELETE_NODE] = gtk_signal_new("delete_node",
												GTK_RUN_FIRST,
												object_class->type,
												GTK_SIGNAL_OFFSET(GtkXmlViewClass, delete_node),
												gtk_marshal_NONE__POINTER,
												GTK_TYPE_NONE, 1,
												GTK_TYPE_POINTER);
	gtk_object_class_add_signals(object_class, xml_view_signals, SIGNAL_NUM);
	klass->select_node = NULL;
}

static void
gtk_xml_view_init( GtkXmlView* view ){
	GtkXmlViewPrivate* priv;
	GTK_WIDGET_SET_FLAGS(GTK_WIDGET(view), GTK_NO_WINDOW);
	view->selected = NULL;
	priv = g_new0( GtkXmlViewPrivate, 1);
	view->private_data = priv;
	view->doc = NULL;
}

GtkWidget*
gtk_xml_view_new(xmlDoc* doc){
	GtkXmlView* view;

	view = gtk_type_new(gtk_xml_view_get_type());
	if( doc == NULL ){
		doc = xmlNewDoc("1.0");
	}
	view->doc = doc;
	gtk_widget_set_usize(GTK_WIDGET(view),200,200);
	return GTK_WIDGET(view);
}
void
gtk_xml_view_update_node( GtkXmlView* view, xmlNode* node ){
	if(!node){
		return;
	}
	gtk_signal_emit( GTK_OBJECT(view), xml_view_signals[UPDATE_NODE], node);
}
void
gtk_xml_view_append_child( GtkXmlView* view, xmlNode* node, xmlNode* parent ){
	GtkXmlViewPrivate* priv = (GtkXmlViewPrivate*)view->private_data;
	if(!node){
		return;
	}
	if(!parent){
		return;
	}
	switch( parent->type ){
		case XML_DOCUMENT_NODE:
			if( view->doc == NULL ){
				view->doc = (xmlDoc*)parent;
			}
			if( xmlDocGetRootElement(view->doc) == NULL ){
				xmlDocSetRootElement(view->doc, node);
			}else{
				if( xmlDocGetRootElement(view->doc) != node ){
					return;
				}
			}
			break;
		case XML_ELEMENT_NODE:
			xmlAddChild(parent, node);
			break;
		default:
			return;
	}
	gtk_signal_emit( GTK_OBJECT(view), xml_view_signals[APPEND_CHILD_NODE], node, parent );
}
void
gtk_xml_view_delete_node( GtkXmlView* view, xmlNode* node ){
	xmlDoc* doc = gtk_xml_view_get_document(view);
	if( !node ){
		return;
	}
	if( gtk_xml_view_get_selected_node(view) == node ){
		gtk_xml_view_unselect_node(view,node);
	}
	gtk_signal_emit( GTK_OBJECT(view), xml_view_signals[DELETE_NODE], node );
	if( node == xmlDocGetRootElement(doc) ){
		view->doc = xmlNewDoc("1.0");
	}
}
void
gtk_xml_view_add_attribute( GtkXmlView* view, xmlNode* node, gchar* attrname, gchar* attrvalue ){
	xmlNewProp(node, attrname, attrvalue);
	gtk_xml_view_update_node( view, node );
}
void
gtk_xml_view_update_attribute( GtkXmlView* view, xmlNode* node, gchar* attrname, gchar* attrvalue ){
	if( xmlSetProp(node,attrname, attrvalue ) ){
		gtk_xml_view_update_node(view, node);
	}
}
void
gtk_xml_view_delete_attribute( GtkXmlView* view, xmlNode* node, gchar* attrname ){
	xmlRemoveProp(xmlSetProp(node, attrname, "tmp"));
	gtk_xml_view_update_node(view, node);
}
void
gtk_xml_view_select_node( GtkXmlView* view, xmlNode* node ){
	if(!node){
		xmlNode* n = gtk_xml_view_get_selected_node(view);
		if( n != NULL ){
			gtk_xml_view_unselect_node(view,n);
		}
		return;
	}
	gtk_signal_emit( GTK_OBJECT(view), xml_view_signals[SELECT_NODE], node);
	view->selected = node;
}
void
gtk_xml_view_unselect_node( GtkXmlView* view, xmlNode* node ){
	if(!node){
		return;
	}
	gtk_signal_emit( GTK_OBJECT(view), xml_view_signals[UNSELECT_NODE]);
	view->selected = NULL;
}
static void
gtk_xml_view_clicked( GtkXmlView* view, xmlNode* node ){
	if(!node){
		return;
	}
	gtk_signal_emit( GTK_OBJECT(view), xml_view_signals[CLICKED]);
}
static void
gtk_xml_view_doubleclicked( GtkXmlView* view, xmlNode* node ){
	if(!node){
		return;
	}
	gtk_signal_emit( GTK_OBJECT(view), xml_view_signals[DOUBLECLICKED]);
}
void
gtk_xml_view_destroy( GtkObject* object ){
	GtkXmlView *view;

	g_return_if_fail(object);
	g_return_if_fail(IS_GTK_XML_VIEW(object));
	view = GTK_XML_VIEW(object);
	g_free(view->private_data);
	if( GTK_OBJECT_CLASS(parent_class)->destroy){
		(*GTK_OBJECT_CLASS(parent_class)->destroy)(object);
	}
}
xmlDoc*
gtk_xml_view_get_document( GtkXmlView *view ){
	return view->doc;
}
xmlNode*
gtk_xml_view_get_selected_node( GtkXmlView* view ){
	return view->selected;
}
